
#include "config.h"

#ifdef PORTABLE_CRYPT

#include <ctype.h>

#define _CRYPT_C_

#define _SHS_include_
#include "crypt.h"

/*
// --------------------------------------------------------------------------
// commonly used througout all algorithms
*/

static unsigned char ascii64[] =        /* 0 ... 63 => ascii - 64 */
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

static void to64(s, v, n)
        char *s;
        unsigned long v;
        int n;
{
        while (--n >= 0) {
                *s++ = ascii64[v&0x3f];
                v >>= 6;
        }
}

char * random_salt(char * salt) {
        salt[0] = ascii64[lrand48() % 64];
        salt[1] = ascii64[lrand48() % 64];
        memset(salt, (int) NULL, CRYPT_MAXSALT + 1);
        return &salt[0];
}

static char *
shs_crypt(pw, pl, sp, sl, passwd)
	register const unsigned char *pw;
        const unsigned int pl;
	register const unsigned char *sp;
        const unsigned int sl;
        char * passwd;
{
        char *p;
	unsigned char	final[SHS_SIZE];
	int i,j;
	SHS_CTX	ctx,ctx1;
	unsigned long l;

	shsInit(&ctx);

	/* The password first, since that is what is most unknown */
	shsUpdate(&ctx,pw,pl);

	/* Then the raw salt */
	shsUpdate(&ctx,sp,sl);

	/* Then just as many characters of the shs(pw,salt,pw) */
	shsInit(&ctx1);
	shsUpdate(&ctx1,pw,pl);
	shsUpdate(&ctx1,sp,sl);
	shsUpdate(&ctx1,pw,pl);
	shsFinal(&ctx1,final);
	for(i = pl; i > 0; i -= SHS_SIZE)
		shsUpdate(&ctx,final,i>SHS_SIZE ? SHS_SIZE : i);

	/* Don't leave anything around in vm they could use. */
	memset(final,0,sizeof final);

	/* Then something really weird... */
	for (j=0,i = pl; i ; i >>= 1)
		if(i&1)
		    shsUpdate(&ctx, final+j, 1);
		else
		    shsUpdate(&ctx, pw+j, 1);

	/* Now make the output string */
	strncpy(passwd, (char *)sp, (int)sl);

	shsFinal(&ctx,final);

	/*
	 * and now, just to make sure things don't run too fast
	 * On a 60 Mhz Pentium this takes 34 msec, so you would
	 * need 30 seconds to build a 1000 entry dictionary...
	 */
	for(i=0;i<1000;i++) {
		shsInit(&ctx1);
		if(i & 1)
			shsUpdate(&ctx1,pw,pl);
		else
			shsUpdate(&ctx1,final,SHS_SIZE);

		if(i % 3)
			shsUpdate(&ctx1,sp,sl);

		if(i % 7)
			shsUpdate(&ctx1,pw,pl);

		if(i & 1)
			shsUpdate(&ctx1,final,SHS_SIZE);
		else
			shsUpdate(&ctx1,pw,pl);
		shsFinal(&ctx1,final);
	}

	p = passwd + strlen(passwd);

	l = (final[ 0]<<16) | (final[ 6]<<8) | final[12]; to64(p,l,4); p += 4;
	l = (final[ 1]<<16) | (final[ 7]<<8) | final[13]; to64(p,l,4); p += 4;
	l = (final[ 2]<<16) | (final[ 8]<<8) | final[14]; to64(p,l,4); p += 4;
	l = (final[ 3]<<16) | (final[ 9]<<8) | final[15]; to64(p,l,4); p += 4;
	l = (final[ 4]<<16) | (final[10]<<8) | final[16]; to64(p,l,4); p += 4;
	l = (final[ 5]<<16) | (final[11]<<8) | final[17]; to64(p,l,4); p += 4;
	l =                   (final[18]<<8) | final[19]; to64(p,l,3); p += 3;

	*p = '\0';

	/* Don't leave anything around in vm they could use. */
	memset(final,0,sizeof final);

	return passwd;
}

char * strcrypt(password, salt)
	char * password;
	char * salt;
{
    char            sbuf[3] = {0,0,0};
    char          * sp,
                  * pw,
                  * ep;
    unsigned int    sl,
                    pl;
    char          * pw_out;

    /* fix signedness */
    sp = salt;
    pw = password;

    pl = strlen((char *) password);

    if (strlen((char *) sp) == 0) {
        sp = random_salt(&sbuf[0]);
        sl = strlen(sp);
    } else {
        for (ep=sp; *ep && *ep != '$'; ep++)
            continue;
        if (!(ep - sp) && *ep == '$') {
            return NULL;
        } else {
            sl = ep - sp;
        }
    }

    pw_out = (char *) malloc(sizeof(char) * CRYPT_OUTPUT_SIZE);

    return shs_crypt(pw, pl, sp, sl, pw_out);
}

int match_crypted(possible, crypted)
     char * possible,
          * crypted;
{
    char * pc;
    char   pbuf[PWD_SIZE];
    int    match;

    pc = strcrypt(possible, crypted);
    strncpy(pbuf, pc, PWD_SIZE - 1);
    free(pc);

    fprintf(stderr, "strcmp('%s', '%s')\n", pbuf, crypted);
    match = !strcmp(pbuf, crypted);

    return match;
}

#endif
